/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Title:        Checklist
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      University Bayreuth
 * @author Dieter Neubacher
 * @version 1.0
 */

package net.metadiversity.diversity.navigator.checklists.dataimport;

// import java.net.URL;          // F�r Datenbank-URL
import java.sql.*;            // JDBC
// import java.lang.*;
// import java.lang.String;
//import java.lang.StringBuffer;
import java.io.FileReader;
// import java.io.StringReader;
// import java.io.BufferedReader;
// import javax.swing.*;

// logging
import org.apache.log4j.Logger;

import net.metadiversity.diversity.navigator.checklists.database.DBChecklist;
import net.metadiversity.diversity.navigator.checklists.ChecklistFileName;
import net.metadiversity.diversity.navigator.checklists.WriteUnicodeFile;
//import net.metadiversity.diversity.navigator.checklists.*;

import net.metadiversity.diversity.navigator.gui.DnOptionPane;
/**
 * Describe class <code>ImportDataFile</code> here.
 *
 * @author <a href="mailto:didi@china.didi.dn">Dieter Neubacher</a>
 * @version 1.0
 */
public class ImportDataFile
{
    static Logger logger = Logger.getLogger( DBChecklist.class.getName() ); 
    
    boolean DEBUG_FLAG = true;
    boolean IMPORT_FORMAT_UTF8 = true;

    // we use BufferdReader for inport file reading
    //
    // posible encodings are:
    //
    // "US-ASCII"         "Seven-bit ASCII, a.k.a. ISO646-US, a.k.a. the Basic Latin block of the Unicode character set"
    // "ISO-8859-1"       "ISO Latin Alphabet No. 1, a.k.a. ISO-LATIN-1"
    // "UTF-8"            "Eight-bit Unicode Transformation Format"
    // "UTF-16BE"         "Sixteen-bit Unicode Transformation Format, big-endian byte order"
    // "UTF-16LE"         "Sixteen-bit Unicode Transformation Format, little-endian byte order"
    // "UTF-16"           "Sixteen-bit Unicode Transformation Format, byte order specified by a mandatory initial byte-order mark (either order accepted on input, big-endian used on output)"



    /**
     * Creates a new <code>ImportDataFile</code> instance.
     *
    */
    public ImportDataFile()
    {  
      // importData( "china.didi.dn", "checklist", "didi", "akiko", "test", 1 );
    }
    /**
     * Creates a new <code>ImportDataFile</code> instance.
     *
     * @param filename a <code>String</code> value
     * @param dataBaseName a <code>String</code> value
     */
    public ImportDataFile( String filename, String dataBaseName )
    {
      //
    }
    //--------------------------------------------------------------
    // Import a new Checklist, only store original lines in database
    //--------------------------------------------------------------
    /**
     * Describe <code>importData</code> method here.
     *
     * @param url a <code>String</code> value
     * @param database a <code>String</code> value
     * @param username a <code>String</code> value
     * @param password a <code>String</code> value
     * @param filename a <code>String</code> value
     * @param index a <code>long</code> value
     * @return a <code>boolean</code> value
     */
//???    public boolean importData( String url, String database, String username, String password, String filename, long index )
    public boolean importData( DBChecklist db, String filename, long index )
    {
        java.io.BufferedReader          input     = null;
        java.util.Vector taxonUnit  = new java.util.Vector();
    
        try
        {
            if( IMPORT_FORMAT_UTF8 )
            {
                try
                {
                    input = new java.io.BufferedReader( new java.io.InputStreamReader( new java.io.FileInputStream( filename ), "UTF-8" ) );
                }
                catch( java.io.UnsupportedEncodingException ex )
                {
                    DnOptionPane.showMessageDialog( null, ex.toString(), "Error" , DnOptionPane.ERROR_MESSAGE );
                    return false;
                }
            }
            else
            {
                input = new java.io.BufferedReader( new FileReader( filename ) );
            }
        }
        catch( java.io.FileNotFoundException ex )
        {
            String msg = "File not found: " + filename + ":\n";
            logger.error( msg );
            DnOptionPane.showMessageDialog( null, msg, "Error", DnOptionPane.ERROR_MESSAGE );
            return false;
        }


        String line;
        int    num = 0;
        try
        {
            boolean ret = true;
            db.checkConnection();
            Connection con = db.connect();
            Statement stmt = con.createStatement();

            while( input.ready() )
            {
                line = input.readLine();
                num++;
                try
                {
                    String cmd = "insert into lines_orginal ( checklist_id, text ) VALUES ( \'" + index + "\',\'" + line.trim() + "\' );";
                    if( DEBUG_FLAG )
                    {
                        logger.debug( cmd );
                    }
                    if( stmt.execute( cmd ) == true )
                    {
                        // there was an error
                        ret = false;
                    }
                }
                catch( Exception ex )
                {
                    logger.error( ex.getMessage(), ex.fillInStackTrace() );            
                }
            }
            return ret;
        }
        catch( java.io.IOException ex )
        {
            logger.error( "Error while reading file: " );
            logger.error( filename );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        catch( java.sql.SQLException ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );            
        }
        return true;
    }
    //--------------------------------------------------------------
    // Read a orginal Checklist, convert to XML-Checklist
    //--------------------------------------------------------------
    public boolean convertToXmlFile( ChecklistFileName checklist ) 
    {
        java.io.BufferedReader          input     = null;
        java.util.Vector taxonUnit  = new java.util.Vector();
      
        try 
        {
          if( IMPORT_FORMAT_UTF8 ) 
          {
              try 
              {
                  input = new java.io.BufferedReader( new java.io.InputStreamReader( new java.io.FileInputStream( checklist.getDefaultFilename()  ), "UTF-8" ) );
              }
              catch( java.io.UnsupportedEncodingException ex ) 
              {
                  DnOptionPane.showMessageDialog( null, ex.toString(), "Error" , DnOptionPane.ERROR_MESSAGE );
                  return false;
              }
          }
          else 
          {
              input = new java.io.BufferedReader( new FileReader( checklist.getDefaultFilename()  ) );
          }
        }
        catch( java.io.FileNotFoundException ex ) 
        {
            String msg = "File not found: " + checklist.getDefaultFilename()  + ":\n";
            logger.error( msg );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
            DnOptionPane.showMessageDialog( null, msg, "Error", DnOptionPane.ERROR_MESSAGE );
            return false;
        }           
        String line;
        String lastName = "";
        int    num = 0;
        int    taxonCounter = 0;
        ParseImportLine pil = new ParseImportLine();
        try 
        {
            boolean ret = true;
          
            while( input.ready() ) 
            {
              // read next input line
              line = input.readLine();
              pil.parseAndConvertToXmlData( line );
              /* ???
              if( pil.parseAndConvertToXmlData( line ) )
              {
                num++;
                try 
                {                  
                  if( DEBUG_FLAG ) 
                  {
                      // System.out.println( cmd );
                  }
                }
                catch( Exception ex ) 
                {
                logger.error( ex.getMessage(), ex.fillInStackTrace() );
                }
              }
               */
            }
            // Write XML-Output to file
             WriteUnicodeFile wuf = new WriteUnicodeFile( checklist.getXmlFilename() , pil.getXmlChecklistData() ); 
            // Write HTML-Output to file
            wuf = new WriteUnicodeFile( checklist.getHtmlFilename(), pil.getHtmlChecklistData() );           
            return ret;
        }
        catch( java.io.IOException ex ) 
        {
            logger.error( "Error while reading file: " );
            logger.error( checklist.getDefaultFilename() );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        return true;
    }  
    //--------------------------------------------------------------
    // Peter Scholz Liste 
    //
    // a??? mark akzeptierten namen
    // s??? mark synonym
    // all other flags are not spicified
    //--------------------------------------------------------------
    public boolean importPeterScholzList( ChecklistFileName checklist ) 
    {
        java.io.BufferedReader          input     = null;
        java.util.Vector taxonUnit  = new java.util.Vector();
      
        try 
        {
          if( IMPORT_FORMAT_UTF8 ) 
          {
              try 
              {
                  input = new java.io.BufferedReader( new java.io.InputStreamReader( new java.io.FileInputStream( checklist.getDefaultFilename()  ), "UTF-8" ) );
              }
              catch( java.io.UnsupportedEncodingException ex ) 
              {
                  DnOptionPane.showMessageDialog( null, ex.toString(), "Error" , DnOptionPane.ERROR_MESSAGE );
                  return false;
              }
          }
          else 
          {
              input = new java.io.BufferedReader( new FileReader( checklist.getDefaultFilename()  ) );
          }
        }
        catch( java.io.FileNotFoundException ex ) 
        {
            String msg = "File not found: " + checklist.getDefaultFilename()  + ":\n";
            logger.error( msg );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
            DnOptionPane.showMessageDialog( null, msg, "Error", DnOptionPane.ERROR_MESSAGE );
            return false;
        }           
        String line;
        String lastName = "";
        int    num = 0;
        int    taxonCounter = 0;
        ParseImportLine pil = new ParseImportLine();
        try 
        {
            boolean ret = true;
          
            while( input.ready() ) 
            {
              // read next input line
              line = input.readLine();
              
              if( line.length() < 5 ) continue;
              if( line.startsWith( "#" ) ) continue;
              String c1 = line.substring( 0, 1 );
              String c2 = "?";        
              String c3 = "z";        
              String c4 = "?";        
              // ??? auct. ???
              /*
              if( line.indexOf( " auct. " ) > 0 )
              {
                  c4 = 
              }
              */
              
              line = line.substring( 5 );   // line without flags
              int pos = 0;              
              // genus ??
              pos = line.indexOf( " " );
              if( pos > 0 )
              { 
                if( line.charAt( pos + 1 ) == '(' )
                {    
                    c3 = "g";
                }
                else if( Character.isUpperCase( line.charAt( pos + 1 ) ) )
                {
                    c3 = "g";
                }
              }
              
              int maxpos = line.indexOf( "#" ); // korrigierte Eintr�ge
              pos = line.indexOf( '#' );
              if( maxpos == -1 )
              {
                  maxpos = line.length();
              }
              pos = line.indexOf( " ssp. " );
              if( pos > 0 && pos < maxpos )
              {    
                c3 = "t";
              }
              pos = line.indexOf( " var. " );
              if( pos > 0  && pos < maxpos )
              {    
                c3 = "r";
              }
              pos = line.indexOf( " f. " );
              if( pos > 0  && pos < maxpos )
              { 
                    // Error if f. & Taylo
                    pos = line.indexOf( " & ", pos );
                    if( pos == -1 || pos > 8 )
                    {    
                        c3 = "f";
                    }
              }
              
              line =  c1 + c2 + c3 + c4 + " " + line;
              
              pil.parseAndConvertToXmlData( line );
              /* ???
              if( pil.parseAndConvertToXmlData( line ) )
              {
                num++;
                try 
                {                  
                  if( DEBUG_FLAG ) 
                  {
                      // System.out.println( cmd );
                  }
                }
                catch( Exception ex ) 
                {
                logger.error( ex.getMessage(), ex.fillInStackTrace() );
                }
              }
               */
            }
            // Write XML-Output to file
             WriteUnicodeFile wuf = new WriteUnicodeFile( checklist.getXmlFilename() , pil.getXmlChecklistData() ); 
            // Write HTML-Output to file
            wuf = new WriteUnicodeFile( checklist.getHtmlFilename(), pil.getHtmlChecklistData() );           
            return ret;
        }
        catch( java.io.IOException ex ) 
        {
            logger.error( "Error while reading file: " );
            logger.error( checklist.getDefaultFilename() );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        return true;
    } 
    //--------------------------------------------------------------
    //
    //--------------------------------------------------------------
    public boolean importLiasNames( ChecklistFileName checklist ) 
    {
        java.io.BufferedReader          input     = null;
        java.util.Vector taxonUnit  = new java.util.Vector();
      
        try 
        {
          if( IMPORT_FORMAT_UTF8 ) 
          {
              try 
              {
                  input = new java.io.BufferedReader( new java.io.InputStreamReader( new java.io.FileInputStream( checklist.getDefaultFilename()  ), "UTF-8" ) );
              }
              catch( java.io.UnsupportedEncodingException ex ) 
              {
                  DnOptionPane.showMessageDialog( null, ex.toString(), "Error" , DnOptionPane.ERROR_MESSAGE );
                  return false;
              }
          }
          else 
          {
              input = new java.io.BufferedReader( new FileReader( checklist.getDefaultFilename()  ) );
          }
        }
        catch( java.io.FileNotFoundException ex ) 
        {
            String msg = "File not found: " + checklist.getDefaultFilename()  + ":\n";
            logger.error( msg );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
            DnOptionPane.showMessageDialog( null, msg, "Error", DnOptionPane.ERROR_MESSAGE );
            return false;
        }           
        String line;
        String lastName = "";
        int    num = 0;
        int    taxonCounter = 0;
        ParseImportLine pil = new ParseImportLine();
        try 
        {
            boolean ret = true;
          
            while( input.ready() ) 
            {
              // read next input line
              line = input.readLine();
              
              String c1 = "a";        
              String c2 = "?";        
              String c3 = "z";        
              String c4 = "?";        
              
              int pos = 0;
              
              // genus ??
              pos = line.indexOf( " " );
              if( pos > 0 )
              { 
                if( line.charAt( pos + 1 ) == '(' )
                {    
                    c3 = "g";
                }
                else if( Character.isUpperCase( line.charAt( pos + 1 ) ) )
                {
                    c3 = "g";
                }
              }
              pos = line.indexOf( " ssp. " );
              if( pos > 0 )
              {    
                c3 = "t";
              }
              pos = line.indexOf( " var. " );
              if( pos > 0 )
              {    
                c3 = "r";
              }
              pos = line.indexOf( " f. " );
              if( pos > 0 )
              { 
                    // Error if f. & Taylo
                    pos = line.indexOf( " & ", pos );
                    if( pos == -1 || pos > 8 )
                    {    
                        c3 = "f";
                    }
              }
              
              line =  c1 + c2 + c3 + c4 + " " + line;
              
              pil.parseAndConvertToXmlData( line );
              /* ???
              if( pil.parseAndConvertToXmlData( line ) )
              {
                num++;
                try 
                {                  
                  if( DEBUG_FLAG ) 
                  {
                      // System.out.println( cmd );
                  }
                }
                catch( Exception ex ) 
                {
                logger.error( ex.getMessage(), ex.fillInStackTrace() );
                }
              }
               */
            }
            // Write XML-Output to file
             WriteUnicodeFile wuf = new WriteUnicodeFile( checklist.getXmlFilename() , pil.getXmlChecklistData() ); 
            // Write HTML-Output to file
            wuf = new WriteUnicodeFile( checklist.getHtmlFilename(), pil.getHtmlChecklistData() );           
            return ret;
        }
        catch( java.io.IOException ex ) 
        {
            logger.error( "Error while reading file: " );
            logger.error( checklist.getDefaultFilename() );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        return true;
    }  
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------  
    java.lang.String getSynonymMarker( java.lang.String lastName, java.lang.String thisName )
    {
        int     cutLastLength = 0;
        int     cutThisLength = 0;
        
        if( lastName.compareTo( thisName ) == 0 )
        {
            return "3";
        }
        
        //----------------------------------------------------------------------
        // er, ra, rum
        //----------------------------------------------------------------------
        if( lastName.endsWith( "er" ) )  { cutLastLength = 2; }
        if( lastName.endsWith( "ra" ) )  { cutLastLength = 2; }
        if( lastName.endsWith( "rum" ) ) { cutLastLength = 3; }
        
        if( thisName.endsWith( "er" ) )  { cutThisLength = 2; }
        if( thisName.endsWith( "ra" ) )  { cutThisLength = 2; }
        if( thisName.endsWith( "rum" ) ) { cutThisLength = 3; }
        
        if( cutLastLength > 0 && cutThisLength > 0 )
        {   
            java.lang.String lastStr = lastName.substring( 0, lastName.length() - cutLastLength );
            java.lang.String thisStr = thisName.substring( 0, thisName.length() - cutThisLength );
            
            if( lastStr.compareTo( thisStr ) == 0 )
            {
                return "3";
            }    
        }
        //----------------------------------------------------------------------
        // a, us, um
        //----------------------------------------------------------------------
        if( lastName.endsWith( "a" ) )  { cutLastLength = 1; }
        if( lastName.endsWith( "us" ) ) { cutLastLength = 2; }
        if( lastName.endsWith( "um" ) ) { cutLastLength = 2; }
        
        if( thisName.endsWith( "a" ) )  { cutThisLength = 1; }
        if( thisName.endsWith( "us" ) ) { cutThisLength = 2; }
        if( thisName.endsWith( "um" ) ) { cutThisLength = 2; }
        
        if( cutLastLength > 0 && cutThisLength > 0 )
        {   
            java.lang.String lastStr = lastName.substring( 0, lastName.length() - cutLastLength );
            java.lang.String thisStr = thisName.substring( 0, thisName.length() - cutThisLength );
            
            if( lastStr.compareTo( thisStr ) == 0 )
            {
                return "3";
            }    
        }
        //----------------------------------------------------------------------
        // is, e
        //----------------------------------------------------------------------        
        cutLastLength = 0;
        cutThisLength = 0;
        
        if( lastName.endsWith( "e" ) )  { cutLastLength = 1; }
        if( lastName.endsWith( "is" ) ) { cutLastLength = 2; }
        
        if( thisName.endsWith( "e" ) )  { cutThisLength = 1; }
        if( thisName.endsWith( "is" ) ) { cutThisLength = 2; }
        
        if( cutLastLength > 0 && cutThisLength > 0 )
        {   
            java.lang.String lastStr = lastName.substring( 0, lastName.length() - cutLastLength );
            java.lang.String thisStr = thisName.substring( 0, thisName.length() - cutThisLength );
            
            if( lastStr.compareTo( thisStr ) == 0 )
            {
                return "3";
            }    
        }                
        return "2";
    }
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args)
    {
        ImportDataFile idf = new ImportDataFile();
        ChecklistFileName cfn = null;
        
        /*
        cfn = new ChecklistFileName( "/home/didi/lias_names/LiasNames_040524/Import/net.lias.names_fhhindex_040523_import.txt.xml" );
        idf.importLiasNames( cfn );
        
        cfn = new ChecklistFileName( "/home/didi/lias_names/LiasNames_040524/Import/net.lias.names_HBEstl_040523_import.txt.xml" );
        idf.importLiasNames( cfn );
       
        cfn = new ChecklistFileName( "/home/didi/lias_names/LiasNames_040524/Import/net.lias.names_liaslight_040523_import.txt.xml" );
        idf.importLiasNames( cfn );
        
        cfn = new ChecklistFileName( "/home/didi/lias_names/LiasNames_040524/Import/net.lias.names_liasmain_040523_import.txt.xml" );
        idf.importLiasNames( cfn );
         */
        // cfn = new ChecklistFileName( "/home/didi/lias_names/LiasNamesScholzChecklistFormat.txt.xml" );
        // cfn = new ChecklistFileName( "/home/didi/lias_names/LiasNamesScholzChecklistFormat.2.txt.xml" );
        cfn = new ChecklistFileName( "/home/didi/lias_names/LiasNamesImportScholzChecklistFormat.txt.xml" );
        // cfn = new ChecklistFileName( "/home/didi/lias_names/test.xml" );
        idf.importPeterScholzList( cfn );
    }   
}
