/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package net.metadiversity.diversity.navigator.db;

import java.util.*;

import java.sql.*;

import net.metadiversity.diversity.navigator.connection.*;

import org.apache.log4j.Logger;

/**
 *
 * @author  didi
 */
public class DBNode 
{
    static Logger logger = Logger.getLogger( DBNode.class ); 
    
    public static final int NODE_TYPE_CAPTION_CONNECTIONS    = 1;
    public static final int NODE_TYPE_DATABASE               = 2;
    public static final int NODE_TYPE_TABLE                  = 3;
    public static final int NODE_TYPE_FIELD                  = 4;

    private static final String POSTGRES_SYSTEM_TABLE_PREFIX_1 = "pga_";
    private static final String POSTGRES_SYSTEM_TABLE_PREFIX_2 = "sql_";
    
    String  name = "";
    int     nodeType = 0;
    Object  data = null;
    Object  data2 = null;
    Vector  childs = null;
    DBNode  parentNode = null;
    ConnectionsPane inPane;

    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public DBNode( ConnectionsPane inPane, DBNode parentNode, int nodeType, String name ) 
    {
	this( inPane, parentNode, nodeType, name, null );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public DBNode( ConnectionsPane inPane, DBNode parentNode, int nodeType, String name, Object data ) 
    {
	this.nodeType   = nodeType;
	this.name       = name;
	this.parentNode = parentNode;
	this.inPane     = inPane;
	this.data       = data;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public String getName()
    {
        return this.name;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public int getNodeType()
    {
        return this.nodeType;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public Object getData()
    {
        return data;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void setData(Object data)
    {
        this.data = data;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public Object getData2()
    {
        return data2;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void setData2(Object data)
    {
        this.data2 = data;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public DBNode getParentNode()
    {
        return this.parentNode;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public ConnectionInfo getConnectionInfo() 
    {
        if( nodeType == NODE_TYPE_CAPTION_CONNECTIONS ) 
        {
            return null;
        }
        DBNode node = this;
        while( node.getNodeType() != NODE_TYPE_DATABASE ) 
        {
            node = node.getParentNode();
        }
        return ( ConnectionInfo ) node.getData();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public Connection getConnection() 
    {
        if( nodeType == NODE_TYPE_CAPTION_CONNECTIONS ) 
        {
            return null;
        }
        DBNode node = this;
        while( node.getNodeType() != NODE_TYPE_DATABASE )
        {   
            node = node.getParentNode();
        }
        if( node.getData2() == null )
        {
            node.load();
        }
        return ( Connection ) node.getData2();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public String getTable() 
    {
        if( nodeType == NODE_TYPE_CAPTION_CONNECTIONS || nodeType == NODE_TYPE_DATABASE ) 
        {
            return null;
        }
        DBNode node = this;
        while( node.getNodeType() != NODE_TYPE_TABLE )
        {
            node = node.getParentNode();
        }
        return node.getName();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    private void load() 
    {
	try 
        {
            if( childs != null) 
            {
                 return;
            }
            Connection con = null;
            switch( this.getNodeType() ) 
            {
		case NODE_TYPE_CAPTION_CONNECTIONS:
                    childs = new Vector();
                    Map connections = ConnectionsManager.getConnections();
                    Iterator iter = connections.keySet().iterator();
                    while( iter.hasNext() )
                    {
                        ConnectionInfo ci = (ConnectionInfo) connections.get( iter.next() );                        
                        childs.add( new DBNode( inPane, this, NODE_TYPE_DATABASE, ci.getName(), ci));
                    }
                    break;
                case NODE_TYPE_DATABASE:
                    childs = new Vector();
                    ConnectionInfo ci = getConnectionInfo();
                    /* ???
                    try 
                    {
                        inPane.getMainWindow().setConnectionStatus("Loading driver ...");
                        Class.forName( ci.getDriver() );
                    } 
                    catch( Exception ex ) 
                    {
                        inPane.getMainWindow().setConnectionError( "Can't load driver: " + ci.getDriver() );
                        // ex.printStackTrace();
                        return;
                    }
                     */
                    try 
                    {
                        inPane.getMainWindow().setConnectionStatus( "Connecting ..." );
                        con = ci.getConnection(); // DriverManager.getConnection( ci.getUrl(), ci.getUser(), ci.getPwd() );
                        setData2( con );
                    } 
                    catch( Exception ex ) 
                    {
                        inPane.getMainWindow().setConnectionError( "Can't connect to: " + ci.getUrl() );
                        logger.error( ex.getMessage(), ex.fillInStackTrace() );
			// ex.printStackTrace();
			return;
                    }					
                    try 
                    {
                        inPane.getMainWindow().setConnectionStatus( "Retrieving tables ..." );
                        DatabaseMetaData MD = con.getMetaData();
                        String[] types = { "TABLE" };
                        ResultSet tables = MD.getTables(null, null, null, types );
                        while( tables.next() ) 
                        {
                            String tableName = tables.getString( "TABLE_NAME" );
                            //
                            // don't add postgres system tables starts with "pga_" and "sql_"
                            if( ! ( tableName.startsWith( POSTGRES_SYSTEM_TABLE_PREFIX_1 ) ||
                                    tableName.startsWith( POSTGRES_SYSTEM_TABLE_PREFIX_2 )
                                  )  
                              )
                            {
                                DBNode child = new DBNode(inPane, this, NODE_TYPE_TABLE, tableName);
                                childs.add( child );
                            }
                        }
                        tables.close();
                    } 
                    catch( Exception ex ) 
                    {
                        inPane.getMainWindow().setConnectionError( "Can't get tables: " + ex.getMessage());
                        logger.error( ex.getMessage(), ex.fillInStackTrace() );
                        ex.printStackTrace();
                        return;
                    }
                    inPane.getMainWindow().setConnectionStatus( "Connected to " + ci.getName() );
                    break;

                case NODE_TYPE_TABLE:

                    childs = new Vector();
                    con = (Connection)getParentNode().getData2();
                    try 
                    {
                        inPane.getMainWindow().setConnectionStatus( "Retrieving fields ..." );
                        DatabaseMetaData MD = con.getMetaData();
                        String tableName = getName();
                        ResultSet columns = MD.getColumns( null, null, tableName, null );
                        while( columns.next() ) 
                        {
                            String columnName = columns.getString( "COLUMN_NAME" );							
                            DBNode child = new DBNode( inPane, this, NODE_TYPE_FIELD, columnName );
                            childs.add( child );
                        }
                        columns.close();
                    } 
                    catch( Exception ex ) 
                    {
                        logger.error( ex.getMessage(), ex.fillInStackTrace() );
                        inPane.getMainWindow().setConnectionStatus( "Can't get fields: " + ex.getMessage() );
                        logger.error( ex.getMessage(), ex.fillInStackTrace() );
                        ex.printStackTrace();
                        return;
                    }
                    inPane.getMainWindow().setConnectionStatus( "Connected to " + getConnectionInfo().getName() );
                    break;
            }
        } 
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
    }
    //----------------------------------------------------------------------
    // Returns true if the receiver is a leaf.
    //----------------------------------------------------------------------
    public boolean isLeaf() 
    {
        return getNodeType() == NODE_TYPE_FIELD;
    }
    //----------------------------------------------------------------------
    // Returns the number of children TreeNodes the receiver contains.
    //----------------------------------------------------------------------
    public int getChildCount() 
    {
        load();
        return childs.size();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public DBNode getChild(int i) 
    {
        load();
        return (DBNode)childs.elementAt(i);
    }
    //----------------------------------------------------------------------
    // Returns the child at the specified index in this node's child array.
    //----------------------------------------------------------------------
    public int getChildIndex(Object ochild) 
    {
        load();
        DBNode child = (DBNode)ochild;
        for( int i=0; i < getChildCount(); i++ ) 
        {
            if( getChild(i).getName().equals(child.getName()) ) 
            {    
                return i;
            }
        }
        return -1;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public String toString() 
    {
        return this.getName();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void reloadChilds() 
    {
        childs = null;
    }
}

