/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * CharacterStateDescription.java
 *
 * Created on 3. M�rz 2004, 15:40
 */

package net.metadiversity.diversity.navigator.db.delta_editor;

//import net.metadiversity.diversity.navigator.gui.delta_editor.DeltaEditorChangeListener;

import org.apache.log4j.Logger;

/**
 *
 * @author  didi
 */
public class CharacterStateDescription extends java.lang.Object
{
    static Logger logger = Logger.getLogger( CharacterStateDescription.class.getName() ); 

    private boolean inappicalibleFlag;
    private boolean calculateFlag;

    private String  tableCellString = null;
    private String  editorString = null;
    private String  deltaString = null;
    
    // type of characterState
    
    private static final int TYPE_UNKNOWN   = 0x00;
    private static final int TYPE_TEXT      = 0x01;
    private static final int TYPE_NUMERICAL = 0x02;
    private static final int TYPE_SELECTION = 0x04;
    
    private int     characterType = TYPE_UNKNOWN;
    
    // Strings used for displaying data
    
    private String min  = null;
    private String low  = null;
    private String mean = null;
    private String high = null;
    private String max  = null;
    
    // Vector to store CharacterStateElements
    private java.util.Vector  states;                                
    
    /** Creates a new instance of CharacterStateDescription */
    public CharacterStateDescription()
    {
        states = new java.util.Vector();
        calculateFlag = true;
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    public boolean add( CharacterStateDescriptionEntry e )
    {
        calculateFlag = true;
        return states.add( e );
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    public boolean isInapplicalible()
    {
        return inappicalibleFlag;
    }    
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    public void setInapplicalible( boolean b )
    {
        inappicalibleFlag = b;
    }
    //--------------------------------------------------------------------------
    // calculate all fields for this CharcaterState
    //--------------------------------------------------------------------------    
    public void calculate()
    {
        // Sort data 
        java.util.Collections.sort( states );
        // variables used for numerical values
        String min  = null;
        String low  = null;
        String mean = null;
        String high = null;
        String max  = null;        
        StringBuffer cellResult = new StringBuffer();
        StringBuffer editResult = new StringBuffer();
        StringBuffer deltaResult = new StringBuffer();
        boolean first = true;
        characterType = TYPE_SELECTION; // ??? TYPE_UNKNOWN;
        
        if( states.size() > 0 )
        {
        }
        for( int i = 0; i < states.size(); i++ )
        {               
            CharacterStateDescriptionEntry entry = (CharacterStateDescriptionEntry) states.get( i );
            if( i == 0 )
            {
//                deltaResult.append( " " + entry.getCID() + "," );
                deltaResult.append( " " + entry.getCID() );
            }
            
            String str = entry.getCS();
            
            if( str.compareTo( "TE" ) == 0 )    // Text element
            {
                str = entry.getText();
                characterType = TYPE_TEXT;
            }
            else if( str.compareTo( "Min" ) == 0 ) // numerical values
            {
                min = String.valueOf( entry.getX() );
                characterType = TYPE_NUMERICAL;
            }
            else if( str.compareTo( "-Low" ) == 0 ) // numerical values
            {
                low = String.valueOf( entry.getX() );
                characterType = TYPE_NUMERICAL;
            }
            else if( str.compareTo( "Mean" ) == 0 ) // numerical values
            {
                mean = String.valueOf( entry.getX() );
                characterType = TYPE_NUMERICAL;
            }
            else if( str.compareTo( "+High" ) == 0 ) // numerical values
            {
                high = String.valueOf( entry.getX() );
                characterType = TYPE_NUMERICAL;
            }
            else if( str.compareTo( "Max" ) == 0 ) // numerical values
            {
                max = String.valueOf( entry.getX() );
                characterType = TYPE_NUMERICAL;
            }

            switch( characterType )
            {
            case    TYPE_NUMERICAL:
                    if( first )
                    {
                        deltaResult.append( "," );
                        first = false;
                    }
                    break;
            case    TYPE_TEXT:
                    if( str != null )
                    {
                        if( ! first )
                        {
                            cellResult.append( "/" );
                            deltaResult.append( "/" );
                        }
                        if( str.trim().length() == 0 )
                        {
                            // empty string text, so mark it as "TE" in GUI
                            cellResult.append( "TE" );
                            deltaResult.append( "<TE>" );
                        }
                        else
                        {
                            cellResult.append( str );                           
                            deltaResult.append( "<" + str  + ">");
                        }    
                    }
                    first = false;
                    break;
            case    TYPE_SELECTION:     // ordert / unordert
                    if( ! first )
                    {
                        cellResult.append( "/" );
                        deltaResult.append( "/" );
                    }
                    else
                    {
                        deltaResult.append( "," );                        
                    }
                    first = false;
                    cellResult.append( entry.getCS() );
                    deltaResult.append( entry.getCS() );
                    break;
            default:
                    if( ! first )
                    {
                        cellResult.append( "/" );
                        deltaResult.append( "/" );
                    }
                    else
                    {
                        cellResult.append( "Unkownd character type: " );
                    }                        
                    first = false;
                    cellResult.append( entry.getCS() );
                    deltaResult.append( entry.getCS() );
                    break;
            }
        }
        
        if( characterType == TYPE_NUMERICAL )
        {
            // format String
            if( min != null )
            {
                String s = "(" + min + "-)";
                cellResult.append( s );
                deltaResult.append( s );
            }
            if( low != null )
            {
                cellResult.append( low );
                deltaResult.append( low );
            }
            if( mean != null )
            {
                if( low != null )
                {
                    cellResult.append( "-" ); 
                    deltaResult.append( "-" ); 
                }
                // result.append( ">" + mean + "<" );
                cellResult.append( mean );
                deltaResult.append( mean );
            }
            if( high != null )
            {
                if( low != null || mean != null )
                {
                    cellResult.append( "-" ); 
                    deltaResult.append( "-" ); 
                }
                cellResult.append( high  );
                deltaResult.append( high  );
            }
            if( max != null )
            {
                String s = "(-" + max + ")";
                cellResult.append( s );
                deltaResult.append( s );
            }
        }
        // set result
        calculateFlag = false;        
        tableCellString = cellResult.toString();
        editorString    = cellResult.toString();
        deltaString     = deltaResult.toString();
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------    
    public String toString()
    {
        if( calculateFlag ) calculate();
        return tableCellString;
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------    
    public String getTableCellText()
    {
        if( calculateFlag ) calculate();
        return editorString;
    }
    //--------------------------------------------------------------------------
    // return: Delta formated character item (used for NaviKey)
    //--------------------------------------------------------------------------    
    public String getItemDeltaFormatedString()
    {
        if( calculateFlag ) calculate();
        return deltaString;            
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------    
    public java.util.Vector getDescriptionStatesVector()
    {
        return states;
    }
}
