/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * DnMenuItem.java
 *
 * Created on 16. March 2003, 04:38
 */

package net.metadiversity.diversity.navigator.gui;

import org.apache.log4j.Logger;

/**
 *
 * @author  didi
 */
public final class DnMenuItem extends javax.swing.JMenuItem
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3833468426667504185L;

    static Logger logger = Logger.getLogger( DnMenuItem.class.getName() );
    
    String              name;
    // ??? javax.swing.Icon    smallIcon;
    
    final static java.lang.String bundleFullPath = "net/metadiversity/diversity/navigator/gui/MenuBundle";
    static java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle( bundleFullPath );
    static java.lang.String iconPath = "/net/metadiversity/diversity/navigator/images/";
    
    /** Creates a new instance of DnMenuItem */
    public DnMenuItem( String entryName ) 
    {
        super( (javax.swing.Action) new DnAction( entryName, null ) );
        
        //------------------------------------
        // set tooltip for this menu item
        //------------------------------------
        try
        {
            String tooltip = bundle.getString( entryName + ".tooltip" );
            if( tooltip != null && tooltip.length() > 0 )
            {    
                setToolTipText( tooltip );
            }
            else
            {
                if( logger.isInfoEnabled() ) logger.info( "no tooltip definiton for menu item " + entryName );
            }
        }
        catch( java.lang.Exception ex )
        {            
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        
        //------------------------------------
        // set mnemonic for this menu item
        //------------------------------------
        try
        {
            String mnemonic = bundle.getString( entryName + ".mnemonic" );
            if( mnemonic != null && mnemonic.length() > 0 )
            {    
                setMnemonic( mnemonic.charAt( 0 ) );
            }
            else
            {
                if( logger.isInfoEnabled() ) logger.info( "no mnemonic definiton for menu item " + entryName );
            }

            // Added by Nik: Enable if i have upgraded to Java 1.4
            /*
            String str = bundle.getString( entryName + ".mnemonicindex" );
            if( str != null )
            {    
                int mnemonicIndex = Integer.parseInt( str );
                if( mnemonicIndex > 0 )
                {    
                    setDisplayedMnemonicIndex( mnemonicIndex );
                }
                else
                {
                    if( logger.isInfoEnabled() ) logger.info( "no mnemonicindex definiton for menu item " + entryName );
                }
            }
             */
        }
        catch( java.lang.Exception ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        //------------------------------------
        // set text for this menu item
        //------------------------------------
        try
        {
            String text = bundle.getString( entryName + ".text" );
            if( text != null && text.length() > 0 )
            {    
                setText( text );
            }
            else
            {
                if( logger.isInfoEnabled() ) logger.info( "no text definiton for menu item " + entryName );
            }
        }
        catch( java.lang.Exception ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        //------------------------------------
        // set icon for this menu item
        //------------------------------------
        try
        {
            String icon = bundle.getString( entryName + ".icon" );
            if( icon != null && icon.length() > 0 )
            {
                setIcon( new javax.swing.ImageIcon( getClass().getResource( iconPath + icon ) ) );
            }
            else 
            {
                if( logger.isInfoEnabled() ) logger.info( "no icon definiton for menu item " + entryName );
            }
        }
        catch( java.lang.Exception ex )
        {
            logger.error( "icon for menu item " + entryName + " not found." );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        //------------------------------------
        // set accelerator for this menu item
        //------------------------------------
        try
        {
            int mask = 0;
            String  command = bundle.getString( entryName + ".accelerator" );
            if( command != null && command.length() > 0 )
            {    
                char    ch = command.charAt( 0 );
                if( command.indexOf( "ctrl" ) > 0 )
                {
                    mask += java.awt.event.InputEvent.CTRL_MASK;
                }
                if( command.indexOf( "alt" ) > 0 )
                {
                    mask += java.awt.event.InputEvent.ALT_MASK;
                }
                if( command.indexOf( "meta" ) > 0 )
                {
                    mask += java.awt.event.InputEvent.META_MASK;
                }
                if( command.indexOf( "shift" ) > 0 )
                {
                    mask += java.awt.event.InputEvent.SHIFT_MASK;
                }
                javax.swing.KeyStroke key = javax.swing.KeyStroke.getKeyStroke( ch, mask );
                setAccelerator( key );
            }
        }
        catch( java.lang.Exception ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
    }
 
    
    public void actionPerformed(java.awt.event.ActionEvent actionEvent) 
    {
        if( logger.isDebugEnabled() ) logger.debug( "DnMenuItem actionPerformd()" );
    }
    
    public void addItemListener(java.awt.event.ItemListener itemListener) {
        super.addItemListener(itemListener);
    }
    
    public void addPropertyChangeListener(java.beans.PropertyChangeListener propertyChangeListener) {
        super.addPropertyChangeListener(propertyChangeListener);
    }
    
    public javax.accessibility.AccessibleContext getAccessibleContext() {
        javax.accessibility.AccessibleContext retValue;
        
        retValue = super.getAccessibleContext();
        return retValue;
    }
    
    public java.awt.Component getComponent() {
        java.awt.Component retValue;
        
        retValue = super.getComponent();
        return retValue;
    }
    
    public java.awt.Font getFont() {
        java.awt.Font retValue;
        
        retValue = super.getFont();
        return retValue;
    }
    
    public Object[] getSelectedObjects() {
        Object[] retValue;
        
        retValue = super.getSelectedObjects();
        return retValue;
    }
    
    public javax.swing.MenuElement[] getSubElements() {
        javax.swing.MenuElement[] retValue;
        
        retValue = super.getSubElements();
        return retValue;
    }
/*    
    public Object getValue(String str) {
    }
*/    
    public boolean imageUpdate(java.awt.Image image, int param, int param2, int param3, int param4, int param5) {
        boolean retValue;
        
        retValue = super.imageUpdate(image, param, param2, param3, param4, param5);
        return retValue;
    }
    
    public boolean isEnabled() {
        boolean retValue;
        
        retValue = super.isEnabled();
        return retValue;
    }
    
    public void menuSelectionChanged(boolean param) {
        super.menuSelectionChanged(param);
    }
/* deprecated    
    public boolean postEvent(java.awt.Event event) {
        boolean retValue;
        
        retValue = super.postEvent(event);
        return retValue;
    }
*/    
    public void processKeyEvent(java.awt.event.KeyEvent keyEvent, javax.swing.MenuElement[] menuElement, javax.swing.MenuSelectionManager menuSelectionManager) {
        super.processKeyEvent(keyEvent, menuElement, menuSelectionManager);
    }
    
    public void processMouseEvent(java.awt.event.MouseEvent mouseEvent, javax.swing.MenuElement[] menuElement, javax.swing.MenuSelectionManager menuSelectionManager) {
        super.processMouseEvent(mouseEvent, menuElement, menuSelectionManager);
    }
    
    public void putValue(String str, Object obj) {
    }
    
    public void remove(java.awt.MenuComponent menuComponent) {
        super.remove(menuComponent);
    }
    
    public void removeItemListener(java.awt.event.ItemListener itemListener) {
        super.removeItemListener(itemListener);
    }
    
    public void removePropertyChangeListener(java.beans.PropertyChangeListener propertyChangeListener) {
        super.removePropertyChangeListener(propertyChangeListener);
    }
    
    public void setEnabled(boolean param) {
        super.setEnabled(param);
    }    
}
